function h_write(hinfo, elev, azim, b, itd, a)
% h_write(hinfo, elev, azim, b [,itd] [,a])
%
% Write HRTF data for a given (elev,azim) location.
%
% HRTF data is specified as stereo B coeffs, itd, and stereo A coeffs,
% where stereo data organized as: row 1 = left chl, row 2 = right chl.
% The coefficient data will be cropped/padded to correspond to
% the alen and blen parameters of the HRTF data set. If itd is not
% specified it is assumed to be 0; if a is not specified, it is assumed
% to be [1 ; 1].
%
% hinfo is HRTF handle returned by h_new or h_open.
%
% Copyright (c) 2006 Wave Arts, Inc.
% Written by Bill Gardner
%

use_text_header = 1;
force_save_hdr = 0;

if (nargin < 5)
	itd = 0;
end
if (nargin < 6)
	a = [1 ; 1];
end

%
% Check arguments
%
azim = round(azim);
if (azim < 0)
	azim = azim + 360;
elseif (azim >= 360)
	azim = azim - 360;
end
if ((azim < 0) | (azim >= 360))
	error(sprintf('azimuth (%f) must be between -360 and 360 degrees',azim));
end

%
% Get globals
%
elevs = h_get_elevs(hinfo);
itds = h_get_itds(hinfo);
fname = h_get_fname(hinfo);
path = h_path(fname);

%
% find closest elevation
%
n_elev = h_get_nelev(hinfo);
for elev_index = 0 : n_elev - 1
	if (elevs(1, elev_index + 1) == elev)
		break;
	elseif (elevs(1, elev_index + 1) > elev)
		if (elev_index > 0) & ...
			(abs(elevs(1, elev_index + 1) - elev) > abs(elevs(1, elev_index) - elev))
			elev_index = elev_index - 1;
		end
		disp(sprintf('using elev %d for %d', elevs(1, elev_index + 1), elev));
		break;
	end
end

%
% Find closest azimuth. Set flip_flag if symmetric data and left hemisphere.
%
n_azim = elevs(2, elev_index + 1);
azim_index = round(azim * n_azim / 360);

sym_flag = h_is_sym(hinfo);
naz = h_naz(n_azim, sym_flag);
if (sym_flag & (azim_index >= naz))
	flip_flag = 1;
   azim_index = n_azim - azim_index;
else
	flip_flag = 0;
end

%
% Open file
%
fid = fopen(path,'r+','ieee-be');
if (fid == -1)
	error(sprintf('cannot open %s',path));
end

%
% Seek to data
%
data_index = 0;
for i = 0 : elev_index - 1
	data_index = data_index + h_naz(elevs(2, i + 1), sym_flag);
end
data_index = data_index + azim_index;
[blen, alen] = h_get_len(hinfo);
data_size = 2 * (blen + alen);
byte_index = data_index * data_size * 4;

%disp(sprintf('elev_index %d azim_index %d data_index %d', elev_index, azim_index, data_index)); 
if fseek(fid, byte_index, -1) == -1
	error(sprintf('cannot seek to offset %d',byte_index));
end

%
% Grab channels.
%
bL = b(1,:);
bR = b(2,:);
aL = a(1,:);
aR = a(2,:);

%
% Swap channels if necessary
%
if flip_flag
	tmp = bL;
	bL = bR;
	bR = tmp;
	tmp = aL;
	aL = aR;
	aR = tmp;
   itd = -itd;
end

%
% Crop/pad.
%
bL = crop(bL, blen);
bR = crop(bR, blen);
aL = crop(aL, alen);
aR = crop(aR, alen);

%
% Write data
%
nw = fwrite(fid, [bL aL bR aR]', 'float32');
if ~(nw == 2 * (blen + alen))
	error('cannot write data');
end

%
% Close file
%
fclose(fid);

%
% Set itd and rewrite matlab data file if itds are used. Saves time
% to not do this unless necessary. We also could do this on file close,
% but this requires that we remember that it is open for writing.
%
itds(data_index + 1) = itd;
h_set_itds(hinfo,itds);
if force_save_hdr | h_is_itdex(hinfo)
	info = hinfo;
	if use_text_header
		h_save_hdr(fname, info, itds, elevs);
	else
		eval(sprintf('save %s info itds elevs',fname));
	end
end
